/*
 ============================================================================
 Name		 : UpdateTimer.cpp
 Copyright   : Symbian Academy, 2007
 Description : Regularly calls the update method of the game engine.
 ============================================================================
 */

#include "UpdateTimer.h"

/**
 * Standard Symbian OS static function for creating an instance of the object.
 * If creation fails, a leave will be sent.
 * The object is left on the cleanup stack.
 * @param aEngine reference to the game engine for callbacks to udpate the world.
 */
CUpdateTimer* CUpdateTimer::NewLC(CEngine* aEngine)
	{
	CUpdateTimer* self = new (ELeave) CUpdateTimer(aEngine);
	CleanupStack::PushL (self);
	self->ConstructL ();
	return self;
	}

/**
 * Standard Symbian OS static function for creating an instance of the object.
 * If creation fails, a leave will be sent.
 * @param aEngine reference to the game engine for callbacks to update the world.
 */
CUpdateTimer* CUpdateTimer::NewL(CEngine* aEngine)
	{
	CUpdateTimer* self = CUpdateTimer::NewLC (aEngine);
	CleanupStack::Pop (self);
	return self;
	}

// Edit 5: Call the base constructor of CActive and assign a standard priority
CUpdateTimer::CUpdateTimer(CEngine* aEngine) :
	
	iEngine(aEngine)
	{
	// Edit 6: Add this active object instance to the Active Scheduler
	
	}

void CUpdateTimer::ConstructL()
	{
	// Edit 7: Create a thread-relative timer. This is done through the
	//         iTimer.CreateLocal()-function of the RTimer class.
	//         The function will return KErrNone when it was successful, or
	//         a standard error code in case of a problem. It's better not to
	//         ignore the return code. The best way to do so is to encapsulate
	//         the function call in a User::LeaveIfError()-statement.
	
	}

CUpdateTimer::~CUpdateTimer()
	{
	// Edit 8: In the destructor, you should always call the Cancel()-method
	//         of the CActive base class, which will call DoCancel() of our
	//         active object.
	
	// Edit 9: The DoCancel()-method (see for the next edit) will cancel the timer.
	//         In the destructor of the class, you should Close() the handle of the
	//         timer. This will also close the associated kernel side object and
	//         should be called when you don't need the RTimer anymore - which 
	//         will surely be the case in the destructor of our active object.
	
	}

/**
 * Request from the application to start an asynchronous process.
 * @param aDelay delay until the asynch. timer event occurs, in microseconds.
 */
void CUpdateTimer::StartProcess(TTimeIntervalMicroSeconds32 aDelay)
	{
	_LIT(KDelayedProcessPanic, "CUpdateTimer");
	// Edit 12: First make sure that the active object is currently inactive.
	//          If it's active, we have found a programming error by the developer and
	//          should raise a panic to exit the application.
	//          An easy way of doing so is to use the macro:
	//          __ASSERT_ALWAYS(<condition that should be the case>, <action if it isn't>);
	//          -> The active object shouldn't be active: !IsActive(). If it is, the action
	//          is to raise a panic: User::Panic(KDelayedProcessPanic, 1);
	//          This uses the error message text defined above.
	

	// If we got here, the ActiveObject is currently idle.
	// Keep a local copy of the delay
	iDelay = aDelay;
	
	// Edit 13: Send a new request to the Asynchronous Service provider,
	//          requesting a callback after the specified amount of time.
	//          The iStatus-variable of this instance is provided so that the 
	//          asynchronous service provider (ASP) can set it to KRequestPending.
	//          Hint: Use the After() method of the timer.
	
	
	// Edit 14: Call CActive::SetActive()
	//          Indicates that the active object has issued a request and that it is now outstanding.
	//          Derived classes must call this function after issuing a request.
	
	}

/**
 * Handles an active objects request completion event.
 * The function is called by the active scheduler when a request completion event occurs, 
 * i.e. after the active schedulers WaitForAnyRequest() function completes.
 */
// Edit 15: Define the RunL()-function


	// Edit 16: The RunL() function indicates that the specified amount of time has
	//           passed. Therefore, update the game engine and redraw the screen.
	//           Uncomment the following line.
	//iEngine->UpdateWorldL ();

	// Edit 17: Start the timer again and set the active object to active. 
	//          This works exactly like in the StartProcess() method above.




/**
 * Increase the speed of the timer -> decrease the interval time between
 * two calls of the timer object.
 * @param aSpeed amount to decrease from the current interval time.
 */
void CUpdateTimer::IncreaseSpeed(TTimeIntervalMicroSeconds32 aSpeed)
	{
	// Decrease the delay between two calls of RunL().
	// TTimeIntervalMicroSeconds32 does not implement the - operator,
	// therefore, the Int-conversion has to be done
	iDelay = iDelay.Int ()- aSpeed.Int ();
	// Make sure the speed doesn't get faster than this
	if ( iDelay.Int ()< 100)
		iDelay = 100;
	}

/**
 * Implements cancellation of an outstanding request.
 * This function is called as part of the active objects Cancel().
 * It must call the appropriate cancel function offered by the active objects 
 * asynchronous service provider.
 * The asynchronous service providers cancel is expected to act immediately.
 * DoCancel() must not wait for event completion; this is handled by Cancel().
 */
// Edit 10: Define the DoCancel()-function


	// Edit 11: Call Cancel() of the iTimer to cancel any outstanding requests
	//           of the RTimer class.
	

